package net.peachjean.mojo.pegdown;

/*
 * Copyright 2001-2005 The Apache Software Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import org.apache.maven.plugin.AbstractMojo;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.project.MavenProject;
import org.apache.maven.project.MavenProjectHelper;
import org.parboiled.common.FileUtils;
import org.pegdown.Extensions;
import org.pegdown.LinkRenderer;
import org.pegdown.ParsingTimeoutException;
import org.pegdown.PegDownProcessor;
import org.pegdown.ToHtmlSerializer;
import org.pegdown.ast.RootNode;
import org.pegdown.ast.VerbatimNode;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.nio.charset.Charset;
import java.util.HashMap;
import java.util.Map;

import net.peachjean.pegdown.uml.PlantUMLVerbatimSerializer;
import net.peachjean.pegdown.uml.VerbatimSerializer;

/**
 * Goal which generates html from input files.
 *
 * The idea is that you can pass a list of documents.  Each document results in a single html file and can be generated from multiple markdown sources.  Optionally, a document
 * may be attached to the project.  It will be attached with type "html"
 *
 * @goal generate
 * 
 * @phase compile
 */
public class GenerateMarkdownMojo
    extends AbstractMojo
{
	/**
	 * @parameter
	 */
	private MDDoc[] docs = new MDDoc[0];

	/**
	 * @parameter
	 */
	private boolean allowUmlContent = true;

	/**
	 * @parameter expression="${project}"
	 */
	private MavenProject project;

	/**
	 * @component
	 */
	private MavenProjectHelper projectHelper;

	/**
	 * @parameter expression="${encoding}" default-value="${project.build.sourceEncoding}"
	 */
	private String encoding;
	private Charset charset;

    public void execute()
        throws MojoExecutionException
    {
	    this.charset = Charset.forName(encoding);
	    for(MDDoc doc: docs)
	    {
		    generatedDocument(doc);
		    boolean attach = isAttached(doc);
		    if(attach)
		    {
			    String classifier = getClassifier(doc);
			    if(classifier != null) {
				    projectHelper.attachArtifact(project, "html", classifier, getGeneratedFile(doc));
			    } else {
				    projectHelper.attachArtifact(project, "html", getGeneratedFile(doc));
			    }
		    }
	    }
    }

	private void generatedDocument(final MDDoc doc) throws MojoExecutionException
	{
		final Map<String, VerbatimSerializer> serializers = new HashMap<String, VerbatimSerializer>();
		if(this.allowUmlContent) {
			PlantUMLVerbatimSerializer.addToMap(serializers);
		}

		PegDownProcessor pegDown = new PegDownProcessor(Extensions.ALL) {
			@Override
			public String markdownToHtml(final char[] markdownSource, final LinkRenderer linkRenderer)
			{
				try {
					RootNode astRoot = parseMarkdown(markdownSource);
					return new ToHtmlSerializer(linkRenderer){
						@Override
						public void visit(final VerbatimNode node)
						{
							if(serializers.containsKey(node.getType()))
							{
								serializers.get(node.getType()).serialize(node, printer);
							}
							else
							{
								super.visit(node);
							}
						}
					}.toHtml(astRoot);
				} catch(ParsingTimeoutException e) {
					return null;
				}

			}
		};

		File generatedFile = getGeneratedFile(doc);
		File generatedFileDirectory = generatedFile.getParentFile();
		boolean mkdirsSucceeded = generatedFileDirectory.mkdirs();
		if(!mkdirsSucceeded && !generatedFileDirectory.exists()) {
			throw new MojoExecutionException("Could not create generated directory " + generatedFileDirectory.getAbsolutePath());
		}
		try
		{
			Writer output = new OutputStreamWriter(new FileOutputStream(generatedFile), charset);
			for(File mdFile: doc.getSources()) {
				String md = FileUtils.readAllText(mdFile, charset);
				output.write(pegDown.markdownToHtml(md));
			}
			output.close();
		}
		catch (IOException e)
		{
			throw new MojoExecutionException("Failed to generated document " + generatedFile.getAbsolutePath(), e);
		}
	}

	private boolean isAttached(final MDDoc doc)
	{
		if(doc.getAttach() != null) {
			return doc.getAttach();
		} else {
			return doc.getClassifier() != null;
		}
	}

	private File getGeneratedFile(final MDDoc doc)
	{
		if(doc.getDestination() != null) {
			return doc.getDestination();
		} else {
			File targetDir = new File(project.getBuild().getDirectory());
			String finalName = project.getBuild().getFinalName();
			String classifier = this.getClassifier(doc);
			if(classifier != null) {
				finalName += "-" + classifier;
			}
			finalName += ".html";
			return new File(targetDir, finalName);
		}
	}

	private String getClassifier(final MDDoc doc)
	{
		return doc.getClassifier();
	}
}
